==> sptk-3.2.6.tbz2: sptk-3.2.6/src/utils/CSmtpConnect.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CSmtpConnect.h  -  description
                             -------------------
    begin                : Jun 25 2003
    copyright            : (C) 2003-2006 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <stdio.h>
#include <sptk3/CBase64.h>
#include <sptk3/CSmtpConnect.h>
#include <sptk3/CException.h>

#include <sptk3/CFileLog.h>

using namespace std;
using namespace sptk;

CSmtpConnect::CSmtpConnect() {
   m_port = 25;
}

CSmtpConnect::~CSmtpConnect() {
   close();
}

#define RSP_BLOCK_SIZE 1024
int CSmtpConnect::getResponse(bool decode) {
   char     readBuffer[RSP_BLOCK_SIZE+1];
   string  longLine;
   bool     readCompleted = false;
   int      rc = 0;

   while (!readCompleted) {
      int len = readLine(readBuffer,RSP_BLOCK_SIZE);
      longLine = readBuffer;
      if (len == RSP_BLOCK_SIZE && readBuffer[RSP_BLOCK_SIZE]!='\n') {
         do {
            len = readLine(readBuffer,RSP_BLOCK_SIZE);
            longLine += readBuffer;
         } while(len == RSP_BLOCK_SIZE);
      }

      if (longLine[3] == ' ') {
         readCompleted = true;
         longLine[3] = 0;
         rc = atoi(longLine.c_str());
      }

      const char * text = longLine.c_str()+4;
      if (decode) {
         longLine = unmime(text);
         m_response.push_back(longLine);
      } else {
         m_response.push_back(text);
      }
   }
   return rc;
}

void CSmtpConnect::sendCommand(string cmd,bool encode) {
   if (!active())
      throw CException("Socket isn't open");
   if (encode)
      cmd = mime(cmd);
   cmd += "\n";
   write(cmd.c_str(),cmd.length());
}

int CSmtpConnect::command(string cmd,bool encodeCommand,bool decodeResponse) {
   m_response.clear();
   if (!active())
      throw CException("Socket isn't open");
   sendCommand(cmd,encodeCommand);
   return getResponse(decodeResponse);
}

string CSmtpConnect::mime(string s) {
   string result;
   CBuffer src;
   src.set(s.c_str(),s.length());
   CBase64::encode(result,src);
   return result;
}

string CSmtpConnect::unmime(string s) {
   CBuffer dest;
   CBase64::decode(dest,s);
   string result(dest.data(),dest.bytes());
   return result;
}

void CSmtpConnect::cmd_login(string user,string password) {
   close();
   open();
   m_response.clear();

   getResponse();

   if (command("HELO") > 251) 
      throw CException(m_response.asString("\n"));

   if (trim(user).length()) {
      int rc = command("AUTH LOGIN",false,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));

      rc = command(user,true,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));

      rc = command(password,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));
   }
}

void CSmtpConnect::cmd_send() {
   sendMessage();
}

void CSmtpConnect::cmd_quit() {
   command("QUIT");
   close();
}

string parseAddress(string fullAddress) {
   int p1 = fullAddress.find("<");
   int p2 = fullAddress.find(">");
   if (p1 < 0 || p2 < 0 || p2 < p1) 
      return fullAddress;
   return trim(fullAddress.substr(p1+1,p2-p1-1));
}

void CSmtpConnect::sendMessage() {
   int rc = command("MAIL FROM:<"+parseAddress(m_from)+">");
   if (rc > 251)
      throw CException("Can't send message:\n"+m_response.asString("\n"));

   string rcpts = m_to + ";" + m_cc + ";" + m_bcc;
   rcpts = replaceAll(rcpts,",",";");
   rcpts = replaceAll(rcpts," ",";");
   CStrings recepients(rcpts,";");
   unsigned cnt = recepients.size();
   for (unsigned i = 0; i < cnt; i++) {
      string address = trim(recepients[i]);
      if (address[0] == 0) continue;
      rc = command("RCPT TO:<"+parseAddress(recepients[i])+">");
      if (rc > 251)
         throw CException("Recepient "+recepients[i]+" is not accepted.\n"+m_response.asString("\n"));
   }

   mimeMessage(m_messageBuffer);
   if (command("DATA") != 354)
      throw CException("DATA command is not accepted.\n"+m_response.asString("\n"));

   sendCommand(m_messageBuffer.data());
   rc = command("\n.");
   if (rc > 251)
      throw CException("Message body is not accepted.\n"+m_response.asString("\n"));
}

==> sptk-3.5.7.05.tbz2: sptk-3.5.7.05/src/utils/CSmtpConnect.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CSmtpConnect.h  -  description
                             -------------------
    begin                : Jun 25 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdio.h>
#include <sptk3/CBase64.h>
#include <sptk3/CSmtpConnect.h>
#include <sptk3/CException.h>

#include <sptk3/CFileLog.h>

using namespace std;
using namespace sptk;

CSmtpConnect::CSmtpConnect() {
   m_port = 25;
}

CSmtpConnect::~CSmtpConnect() {
   close();
}

#define RSP_BLOCK_SIZE 1024
int CSmtpConnect::getResponse(bool decode) {
   char     readBuffer[RSP_BLOCK_SIZE+1];
   string   longLine;
   bool     readCompleted = false;
   int      rc = 0;

   while (!readCompleted) {
      uint32_t len = readLine(readBuffer,RSP_BLOCK_SIZE);
      longLine = readBuffer;
      if (len == RSP_BLOCK_SIZE && readBuffer[RSP_BLOCK_SIZE]!='\n') {
         do {
            len = readLine(readBuffer,RSP_BLOCK_SIZE);
            longLine += readBuffer;
         } while(len == RSP_BLOCK_SIZE);
      }

      if (longLine[3] == ' ') {
         readCompleted = true;
         longLine[3] = 0;
         rc = atoi(longLine.c_str());
      }

      const char * text = longLine.c_str()+4;
      if (decode) {
         longLine = unmime(text);
         m_response.push_back(longLine);
      } else {
         m_response.push_back(text);
      }
   }
   return rc;
}

void CSmtpConnect::sendCommand(string cmd,bool encode) {
   if (!active())
      throw CException("Socket isn't open");
   if (encode)
      cmd = mime(cmd);
   cmd += "\n";
   write(cmd.c_str(),(uint32_t)cmd.length());
}

int CSmtpConnect::command(string cmd,bool encodeCommand,bool decodeResponse) {
   m_response.clear();
   if (!active())
      throw CException("Socket isn't open");
   sendCommand(cmd,encodeCommand);
   return getResponse(decodeResponse);
}

string CSmtpConnect::mime(string s) {
   string result;
   CBuffer src;
   src.set(s.c_str(),(uint32_t)s.length());
   CBase64::encode(result,src);
   return result;
}

string CSmtpConnect::unmime(string s) {
   CBuffer dest;
   CBase64::decode(dest,s);
   string result(dest.data(),dest.bytes());
   return result;
}

void CSmtpConnect::cmd_login(string user,string password) {
   close();
   open();
   m_response.clear();

   getResponse();

   if (command("HELO") > 251) 
      throw CException(m_response.asString("\n"));

   if (trim(user).length()) {
      int rc = command("AUTH LOGIN",false,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));

      rc = command(user,true,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));

      rc = command(password,true);
      if (rc > 432)
         throw CException(m_response.asString("\n"));
   }
}

void CSmtpConnect::cmd_send() {
   sendMessage();
}

void CSmtpConnect::cmd_quit() {
   command("QUIT");
   close();
}

string parseAddress(string fullAddress) {
   size_t p1 = fullAddress.find("<");
   size_t p2 = fullAddress.find(">");
   if (p1 == string::npos || p2 == string::npos || p2 < p1) 
      return fullAddress;
   return trim(fullAddress.substr(p1+1,p2-p1-1));
}

void CSmtpConnect::sendMessage() {
   int rc = command("MAIL FROM:<"+parseAddress(m_from)+">");
   if (rc > 251)
      throw CException("Can't send message:\n"+m_response.asString("\n"));

   string rcpts = m_to + ";" + m_cc + ";" + m_bcc;
   rcpts = replaceAll(rcpts,",",";");
   rcpts = replaceAll(rcpts," ",";");
   CStrings recepients(rcpts,";");
   uint32_t cnt = (uint32_t)recepients.size();
   for (uint32_t i = 0; i < cnt; i++) {
      string address = trim(recepients[i]);
      if (address[0] == 0) continue;
      rc = command("RCPT TO:<"+parseAddress(recepients[i])+">");
      if (rc > 251)
         throw CException("Recepient "+recepients[i]+" is not accepted.\n"+m_response.asString("\n"));
   }

   mimeMessage(m_messageBuffer);
   if (command("DATA") != 354)
      throw CException("DATA command is not accepted.\n"+m_response.asString("\n"));

   sendCommand(m_messageBuffer.data());
   rc = command("\n.");
   if (rc > 251)
      throw CException("Message body is not accepted.\n"+m_response.asString("\n"));
}
